@@ -1974,6 +1974,7 @@ int write_index(struct index_state *istate, int newfd)
     struct cache_entry **cache = istate->cache;
     int entries = istate->cache_nr;
     SeafStat st;
+    const char *prev_name = NULL;
     int ret = 0;
 
     memset (&info, 0, sizeof(info));
@@ -2005,6 +2006,11 @@ int write_index(struct index_state *istate, int newfd)
         struct cache_entry *ce = cache[i];
         if (ce->ce_flags & CE_REMOVE)
             continue;
+        // skip duplicate cache.
+        if (prev_name && g_strcmp0 (ce->name, prev_name) ==0) {
+            continue;
+        }
+        prev_name = ce->name;
         /* if (!ce_uptodate(ce) && is_racy_timestamp(istate, ce)) */
         /*     ce_smudge_racily_clean_entry(ce); */
         if (ce_write_entry2(&info, newfd, ce) < 0) {
